----------------------------------------------------------------------------------------------------------------------------------------------------
-- WhatRepRecipes - View vendor sell values for all crafted items and reputation requirements, source, price and mats for all unskilled recipe ranks
----------------------------------------------------------------------------------------------------------------------------------------------------
-- Modules/GUI_debuglog_fmt_recipes.lua - Content for Recipe List
--
-- Author: Expelliarm5s / July 2021 / All Rights Reserved
--
-- Version 1.2.8
-------------------------------------------------------------------------------------------------------
-- luacheck: ignore 212
-- luacheck: globals TradeSkillFrame
-- luacheck: max_line_length 250

local addonName, addon = ...
local GUI = addon:GetModule("GUI")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local LibQTip = LibStub("LibQTip-1.0")
local private = {}
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Settings

-- Recipes Debug Log Format
private.recipes = {}
private.recipes.colNames = { L["No"], " " .. L["RecipeID"], L["ItemID"], L["Lvl"], L["Num"], " " .. L["Item"], L["Reagent"], L["Count"], L["Worth"], L["VendorSell"], L["Int"], }
private.temp = 1             -0.06    -0.08                 -0.08        -0.05     -0.05                      -0.08          -0.05       -0.1        -0.1            -0.001
private.recipes.colWidth = { -0.06,   -0.08,                -0.08,       -0.05,    -0.05,    private.temp,     0.08,         -0.05,      -0.1,       -0.1,            0.001, }
private.recipes.colFlex = { "flex",  "flex",               "flex",       "flex",   "flex",   "search",         "drop",       "flex",     "flex",     "flex",          "flex", }
private.recipes.statusText = {
			L["Sort by No"],
			L["Sort by RecipeID"],
			L["Sort by ItemID"],
			L["Sort by Item Level"],
			L["Sort by number of craftable items"],
			L["Sort by Item"],
			L["Sort by Reagent"],
			L["Sort by Main Mat Count"],
			L["Sort by Main Mat Worth"],
			L["Sort by VendorSell"],
			L["Sort by Message"],
		}
private.recipes.GetSTData = function(...) return private.GetSTData_recipes(...) end

------------------------------------------------------------------------------
-- Scrolling Table Data

-- Return Scrolling Table Data
function private.GetSTData_recipes(a, flex, filter)
	GUI:DebugPrintf("GetSTData_recipes(%s, %s, %s)", tostring(a), tostring(flex), tostring(filter))

	-- generate content only if visible
	if not GUI.display or GUI.view ~= a then
		GUI:DebugPrintf("... nothing done.")
		return
	end

	local content = {}

	-- generate content only if log data present
	addon.debuglog = addon.debuglog or {}
	if not addon.debuglog[a] then
		return content, flex
	end

	-- generate content
	local colHasContent = {}
	local k = 1
	for _, rdata in pairs(addon.debuglog[a]) do
		local filterMatch = nil  -- = nil -> no filter input = true -> filtered

		local c1, c2, c3, _, c5, c6, _, c8, c9, c10, c11 = strmatch(rdata.m or "", "1=(.*)# 2=(.*)# 3=(.*)# 4=(.*)# 5=(.*)# 6=(.*)# 7=(.*)# 8=(.*)# 9=(.*)# 10=(.*)# 11=(.*)#")

		local data = {}
		data[2] = c1 or "" -- RecipeID
		data[3] = addon.Item:GetItemID(c5) or "" -- ItemID
		data[4] = c8 or "" -- effLvl
		data[5] = c2 or "" -- Num Available
		data[6] = c5 or "" -- ItemLink
		data[7] = c3 or "" -- ReagentID
		data[8] = c9 or "" -- main mats ItemID
		data[9] = c10 or "" -- main mats count
		data[10] = c11 or "" -- worth per main mats
		data[11] = c6 or "" -- VendorSell
		data[12] = rdata.m or ""

		-- check dropdown
		for j, col in ipairs(flex) do
			colHasContent[j] = colHasContent[j] or data[j]
			if col == "drop" then
				if filter and filter[j] and filter[j] ~= "" then
					if (filterMatch == nil or filterMatch) then
						if data[j] and filter[j] == data[j] then
							filterMatch = true
						else
							filterMatch = false
						end
					end
				end
			end
		end

		-- check search
		for j, col in ipairs(flex) do
			if col == "search" then
				if filter and filter[j] and filter[j] ~= "" then
					if (filterMatch == nil or filterMatch) then
						-- search in all column data
						local found = false
						for m, _ in pairs(data) do
							if data[m] and type(data[m]) == "string" and strfind(strlower(data[m]), strlower(filter[j])) then
								filterMatch = true
								found = true
							end
						 end
						if not found then
							filterMatch = false
						end
					end
				end
			end
		end

		if filterMatch or filterMatch == nil then
			-- convert data into final widget data
			local en = {}
			en.data = {}

			local q = "ffffff"
			en.data[1] = {k, k}
			en.data[2] = {data[2], tonumber(data[2]) or 0}
			addon.recipeColNumRecipeID = 2
			en.data[3] = {data[3], tonumber(data[3]) or 0}
			en.data[4] = {tonumber(data[4]) or "", tonumber(data[4]) or 0}
			en.data[5] = {tonumber(data[5]) or "", tonumber(data[5]) or 0}
			addon.recipeColNumItemLink = 6
			en.data[6] = {data[6], addon.Item:GetItemName(data[6]) or ""}
			en.data[7] = {data[7], data[7] or ""}
			en.data[10] = {addon.Item:FromCopper(tonumber(data[11]) or 0), tonumber(data[11]) or 0}

			if addon.isDebug then
				en.data[11] = {"|cff" .. q .. (data[12] or "") .. "|r", data[12] or ""}
			else
				en.data[11] = {"", ""}
			end

			addon.recipeColNumWorth = 9
			if addon.db.global.isExp then
				en.data[8] = {tonumber(data[9]) or "", tonumber(data[9]) or 0}
				local worth = tonumber(data[10]) or 0
				if worth and worth > 0 then
					local temp = addon.db.global.discardCopper
					addon.db.global.discardCopper = true
					worth = addon.Item:FromCopper(worth)
					addon.db.global.discardCopper = temp
				else
					worth = ""
				end
				en.data[9] = {worth, tonumber(data[10]) or 0}
			else
				en.data[8] = {"", ""}
				en.data[9] = {"", ""}
			end

			tinsert(content, en)
			k = k + 1
		end
	end

	-- disable dropdown if not present
	for j, col in ipairs(flex) do
		if col == "drop" and not colHasContent[j] then
			flex[j] = ""
		end
	end

	return content, flex
end

-- Provide left + right click on table frame
GUI.debugLog_OnClick = GUI.debugLog_OnClick or {}
function GUI.debugLog_OnClick.recipes(btn, data, button)
	GUI:DebugPrintf("OnClick: %s %s %s ", tostring(btn), tostring(data), tostring(button))
	if button == "RightButton" then
		if data and data.data and data.data[addon.recipeColNumItemLink] and data.data[addon.recipeColNumItemLink][1] then
			addon:Printf(addon.Item:EscapedLink(data.data[addon.recipeColNumItemLink][1]))
		end
	end
	if button == "LeftButton" then
		if data and data.data and data.data[addon.recipeColNumRecipeID] and data.data[addon.recipeColNumRecipeID][2] then
			if tonumber(data.data[addon.recipeColNumRecipeID][2]) then
				if TradeSkillFrame and TradeSkillFrame.RecipeList and TradeSkillFrame.RecipeList.SelectedAndForceRecipeIDIntoView then
					TradeSkillFrame.RecipeList:SelectedAndForceRecipeIDIntoView(data.data[addon.recipeColNumRecipeID][2])
				end
			end
		end
	end
end

-- Provide tooltip on content button
GUI.debugLog_OnEnter = GUI.debugLog_OnEnter or {}
function GUI.debugLog_OnEnter.recipes(btn, data)
	GUI:DebugPrintf("OnEnter: %s %s %s ", tostring(btn), tostring(data))

	if btn.btnCol and btn.btnCol == addon.recipeColNumItemLink then
		local link
		if data and data.data and data.data[addon.recipeColNumItemLink] then
			link = data.data[addon.recipeColNumItemLink][1]
		end

		if link then
			GameTooltip:SetOwner(btn, "ANCHOR_TOPRIGHT")
			pcall(GameTooltip.SetHyperlink, GameTooltip, link)
		end
	elseif btn.btnCol and btn.btnCol == addon.recipeColNumWorth then
		local tooltip = LibQTip:Acquire(addon)
		if tooltip then
			tooltip:SmartAnchorTo(GUI.container.frame)
			tooltip:SetAutoHideDelay(0.1, btn)
			tooltip:EnableMouse(true)

			tooltip:Clear()
			tooltip:SetColumnLayout(2,"LEFT", "RIGHT")

			local line = tooltip:AddLine()
			tooltip:SetCell(line, 1, "|cfffed100" .. L["Experimental!"] .. "|r", nil, "CENTER", 2)
			-- tooltip:AddSeparator()

			line = tooltip:AddLine()
			tooltip:SetCell(line, 1, L["This column shows the value per weave / bar etc.|n"] ..
				L["Additional vendor or trade skill mats are not yet deducted.|n"] ..
				L["Just use this as a guidance to which recipe may be worth more than another."])

			tooltip:Show()
		end
	end
end

-- Hide Tooltip
GUI.debugLog_OnLeave = GUI.debugLog_OnLeave or {}
function GUI.debugLog_OnLeave.recipes(btn, data)
	GUI:DebugPrintf("OnLeave: %s %s %s ", tostring(btn), tostring(data))

	GUI:SetStatusLine()

	if btn.btnCol and btn.btnCol == addon.recipeColNumItemLink then
		GameTooltip:Hide()
	end
end

------------------------------------------------------------------------------
-- Create Recipe Tab

function GUI.CreateRecipesTab(a, additionalUICB, forced)
	GUI:DebugPrintf("CreateRecipesTab(%s)", tostring(a))

	if not a or type(a) ~= "string" then
		-- should not never happen!
		GUI:DebugPrintf("ERR~  missing tab name!")
		return
	end

	if GUI.guiTabs.select[a] and not forced then
		GUI:DebugPrintf("Tab %s exists.", tostring(a))
		return
	end

	-- experimental work in progress
	if addon.db.global.isExp then
		private.recipes.colNames = { L["No"], " " .. L["RecipeID"], L["ItemID"], L["Lvl"], L["Num"], " " .. L["Item"], L["Reagent"], L["Count"], L["Worth"], L["VendorSell"], L["Int"], }
		private.temp = 1             -0.06    -0.08                 -0.08        -0.05     -0.05                      -0.08          -0.05       -0.1        -0.1            -0.001
		private.recipes.colWidth = { -0.06,   -0.08,                -0.08,       -0.05,    -0.05,    private.temp,     0.08,         -0.05,      -0.1,       -0.1,            0.001, }
	else
		private.recipes.colNames = { L["No"], " " .. L["RecipeID"], L["ItemID"], L["Lvl"], L["Num"], " " .. L["Item"], L["Reagent"], L["Count"], L["Worth"], L["VendorSell"], L["Int"], }
		private.temp = 1             -0.06    -0.08                 -0.08        -0.05     -0.05                      -0.08          -0.001      -0.001      -0.1            -0.001
		private.recipes.colWidth = { -0.06,   -0.08,                -0.08,       -0.05,    -0.05,    private.temp,     0.08,         -0.001,     -0.001,     -0.1,            0.001, }
	end

	-- create a new log format
	if not GUI.DebugLog_IsFormatRegistered("recipes") or forced then
		GUI.DebugLog_RegisterFormat("recipes", private.recipes)
	end

	-- adjust default data widget parameters
	addon.db.global.showExport = true
	GUI.debugLog_defaultNoSelfTab = true
	GUI.debugLog_defaultTextSize = tonumber(addon.db.global.textSize) or 12
	GUI.debugLog_defaultHeight = 500
	GUI.debugLog_defaultWidth = 800

	if addon.db.global.vendorSellSort then
		GUI.debugLog_defaultSortTitle = -10 -- change this to: vendorsell column number
	else
		GUI.debugLog_defaultSortTitle = 1
	end

	GUI.debugLog_defaultFrameWidget = "WRRFrame"
	GUI.debugLog_defaultTabGroupWidget = "WRRTabGroup"
	GUI.DebugLog_SetWidgets({ data = "WRRData"})

	-- create tab as custom debug log
	GUI.DebugLog_CreateTab(a, "recipes", nil, additionalUICB)
	-- no need to have a timer
	GUI.guiTabs.timer[a] = nil

	GUI:DebugPrintf("WARN~Log for '" .. a .. "' created")

	-- no implicit Load()
	-- GUI:Load(a)
	GUI:DebugPrintf("  <<CreateRecipesTab(%s)", tostring(a))
end

-- EOF
